// Clase base de datos tutorías (BDTutorias). Métodos:
//    constructor BDTutorias: carga el controlador JDBC y llama conectar.
//    conectar: realiza la conexión con la base de datos.
//    obtenerTabla: obtiene todas las filas de la tabla.
//    tutoriasDisponibles: obtiene las tutorías no concertadas.
//    insertarEnCitas: inserta una fila en la tabla citas.
//    borrarCitas: borra la tabla citas.
//    obtenerIdProfesor: obtener el id del profesor "nombre"
//    obtenerIdTutoria: obtener el id de una determinada tutoría
//    noEstaReservada: devuelve true si la tutoría no está reservada
//    cerrarConexion: cerrar la conexión con la base de datos.

import java.sql.*;

public class BDTutorias
{
  private Connection conexión;
  private Statement sentenciaSQL;
  private ResultSet cdr; // conjunto de resultados
  
  public BDTutorias() throws ClassNotFoundException,
      SQLException, InstantiationException,
      IllegalAccessException
  {
    // Cargar el controlador JDBC
    String controlador = "com.mysql.jdbc.Driver";
    Class.forName(controlador).newInstance();
    conectar(); // conectar con la fuente de datos
  }
  
  public void conectar() throws SQLException
  {
    String URL_bd = "jdbc:mysql://127.0.0.1:3306/bd_tutorias";
    String usuario = "";
    String contraseña = "";
    // Conectar con la BD
    conexión = DriverManager.getConnection(
        URL_bd, usuario, contraseña);
    // Crear una sentencia SQL
    sentenciaSQL = conexión.createStatement(
                       ResultSet.TYPE_SCROLL_INSENSITIVE,
                       ResultSet.CONCUR_UPDATABLE);
    System.out.println("\nConexión realizada con éxito.\n");
  }
  
  public void cerrarConexion() throws java.sql.SQLException
  {
    if (cdr != null) cdr.close();
    if (sentenciaSQL != null) sentenciaSQL.close();
    if (conexión != null) conexión.close();
  }

  public ResultSet obtenerTabla(String tabla)
      throws SQLException
  {
    cdr = sentenciaSQL.executeQuery("SELECT * FROM " + tabla);
    return cdr;
  }
  
  public ResultSet tutoriasDisponibles()
      throws SQLException
  {
    cdr = sentenciaSQL.executeQuery(
         "SELECT tutorias.id_tutoria, profesores.profesor, " + 
                 "tutorias.dia, tutorias.hora " +
            "FROM profesores, tutorias " +
            "WHERE profesores.id_profesor=tutorias.id_profesor " +
            "AND tutorias.id_tutoria NOT IN " +
            "(SELECT id_tutoria FROM citas)");
    return cdr;
  }
  
  public void insertarEnCitas(int id_tutoria, String alumno, String asunto)
      throws SQLException
  {
    sentenciaSQL.executeUpdate("INSERT INTO citas" +
      " VALUES (" + id_tutoria + ", '" + alumno + "', '" + asunto + "')"
     );
  }

  public void borrarCitas() throws SQLException
  {
    sentenciaSQL.executeUpdate("DELETE FROM citas");
  }
  
  public int obtenerIdProfesor(String nombre) throws SQLException
  {
    cdr = sentenciaSQL.executeQuery(
        "SELECT * FROM profesores" +
           " WHERE profesor = " + "'" + nombre + "'");
    if (cdr.isBeforeFirst())
    {
      cdr.first();
      return cdr.getInt("id_profesor");
    }
    else
      return -1;
  }
  
  public int obtenerIdTutoria(int id_prof, String d, String h)
      throws SQLException
  {
    cdr = sentenciaSQL.executeQuery(
        "SELECT * FROM tutorias" +
           " WHERE id_profesor = " + id_prof + 
           " AND dia = " + "'" + d + "'" +
           " AND hora = " + "'" + h + "'");
    if (cdr.isBeforeFirst())
    {
      cdr.first();
      return cdr.getInt("id_tutoria");
    }
    else
      return -1;
  }
  
  public boolean noEstaReservada(int idT)
      throws SQLException
  {
    cdr = sentenciaSQL.executeQuery(
        "SELECT id_tutoria FROM citas WHERE id_tutoria = " + idT);
    if (cdr.isBeforeFirst())
      return false; // está reservada
    else
      return true; // no está reservada
  }
}
