/*
 * SvTutorias.java
 *
 * autor Fco. Javier Ceballos
 */

import java.io.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;

public class SvTutorias extends HttpServlet
{
  private BDTutorias BD;

  public void init(ServletConfig config) throws ServletException
  {
    super.init(config);
    try
    {
      BD = new BDTutorias();
    }
    catch(ClassNotFoundException e)
    { 
      System.out.println("Clase no encontrada. " + e.getMessage());
    }
    catch(InstantiationException e)
    { 
      System.out.println("Objeto no creado. " + e.getMessage());
    }
    catch(IllegalAccessException e)
    { 
      System.out.println("Acceso ilegal. " + e.getMessage());
    }
    catch(SQLException e)
    { 
      System.out.println("Excepcin SQL. " + e.getMessage());
    }
  }

  // Manipular la peticin enviada por el cliente
  // utilizando el atributo method=post.
  protected void doPost(HttpServletRequest request,
                        HttpServletResponse response)
  throws ServletException, IOException
  {
    // Tipo de la respuesta que ser enviada al cliente
    response.setContentType("text/html");

    // Obtener el objeto 'PrintWriter' para devolver la respuesta
    PrintWriter out = response.getWriter();
    
    // Tomar los datos recibidos del cliente y escribirlos
    // en la base de datos.
    try
    {
      // Obtener los datos enviados por el cliente.
      String profesor = request.getParameter("profesor");
      String da = request.getParameter("da");
      String hora = request.getParameter("hora");
      String alumno = request.getParameter("alumno");
      String asunto = request.getParameter("asunto");
      int id_profesor = -1, id_tutoria = -1;
      id_profesor = BD.obtenerIdProfesor(profesor);
      if (id_profesor != -1)
      {
        id_tutoria = BD.obtenerIdTutoria(id_profesor, da, hora);
        if (id_tutoria != -1 && BD.noEstaReservada(id_tutoria))
        {
          // Insertar una fila en la tabla citas
          BD.insertarEnCitas(id_tutoria, alumno, asunto);

          // Responder al cliente
          out.println("<html>");
          out.println("<title>Respuesta a la solicitud</title>");
          out.println("Su peticin ha sido registrada<br>Un saludo");
          out.println("</html>");
          return;
        }
      }
      out.println("No se pudo atender su solicitud. Elija otra " +
                  "opcin, si la hay, entre las siguientes:<br><br>");
      mostrarTutoriasDisponibles(out);
    }
    catch(SQLException e)
    {
      out.println("Excepcin SQL. " + e.getMessage());
    }
    // Cerrar el flujo
    out.close();
  }
  
  // Devuelve un formulario html con las tutoras disponibles.
  public void mostrarTutoriasDisponibles(PrintWriter out)
  {
    try
    {
      ResultSet cdr = BD.tutoriasDisponibles();
      // Mostrar el conjunto de resultados obtenido
      out.println("<html><head><title>Tutoras</title></head>" +
      "<body>");
      // Tabla
      out.println("<table width=100% border=1>");
      // Cabeceras
      out.println("<tr>" +
                    "<th>Da/hora</th><th>Con el profesor</th>" +
                  "</tr>");
      // Filas
      while (cdr.next())
      {
        out.println(
         "<tr>" +
           "<td width=30%>" + cdr.getString("dia") +
                  " a las " + cdr.getString("hora") + "</td>" +
           "<td width=50%>" + cdr.getString("profesor") + "</td>" +
         "</tr>");
      } 
      out.println("</table></body></html>");
    }
    catch(SQLException e)
    {
      out.println("Excepcin SQL. " + e.getMessage());
    }
  }
  
  public void destroy()
  {
    try
    {
      BD.cerrarConexion();
    }
    catch(SQLException ignorada){}
  }

  // Devuelve una descripcin breve.
  public String getServletInfo()
  {
    return "Servlet SvTutorias";
  }
}
