// Clase base de datos alumnos (BDAlumnos). Métodos:
//    constructor BDAlumnos: carga el controlador JDBC y conectar.
//    conectar: realiza la conexión con la base de datos.
//    tablas: devuelve el conjunto de tablas disponibles en la BD.
//    obtenerTabla: obtiene todas las filas de la tabla.
//    buscarFilasEnAlumnos: obtiene determinadas filas de la tabla alumnos.
//    insertarFilaEnAlumnos: inserta una fila en la tabla alumnos.
//    borrarFilaEnAlumnos: borra una fila de la tabla alumnos.
//    mostrarTabla: visualiza todas las filas de una tabla.
//    mostrarFilasDeAlumnos: visualiza determinadas filas de la tabla alumnos.
//    mostrarFilaActual: muestra la fila actual de un ResultSet.
//    siguiente: mover el cursor a la fila siguiente.
//    anterior: mover el cursor a la fila anterior.
//    primero: mover el cursor a la primera fila.
//    último: mover el cursor a la última fila.
//    cerrarConexion: cerrar la conexión con la base de datos.

package Cap07.Resueltos.Base_de_datos;

public class BDAlumnos
{
  private java.sql.Connection conexión;
  private java.sql.Statement sentenciaSQL;
  private java.sql.ResultSet cdr; // conjunto de resultados
  
  public BDAlumnos() throws ClassNotFoundException, java.sql.SQLException,
      InstantiationException, IllegalAccessException
  {
    // Cargar el controlador JDBC
    String controlador = "sun.jdbc.odbc.JdbcOdbcDriver";
    Class.forName(controlador).newInstance();
    conectar(); // conectar con la fuente de datos
  }
  
  public void conectar() throws java.sql.SQLException
  {
    String URL_bd = "jdbc:odbc:bd-alumnos";
    String usuario = "";
    String contraseña = "";
    // Conectar con la BD
    conexión = java.sql.DriverManager.getConnection(
        URL_bd, usuario, contraseña);
    // Crear una sentencia SQL
    sentenciaSQL = conexión.createStatement(
                       java.sql.ResultSet.TYPE_SCROLL_INSENSITIVE,
                       java.sql.ResultSet.CONCUR_UPDATABLE);
    System.out.println("\nConexión realizada con éxito.\n");
    // Mostrar las tablas de la base de datos
    System.out.println("Tablas de la base de datos: ");
    String[] tabla = tablas();
    for (int i = 0; i < tabla.length; ++i)
      System.out.println(tabla[i]);
  }
  
  public void cerrarConexion() throws java.sql.SQLException
  {
    if (cdr != null) cdr.close();
    if (sentenciaSQL != null) sentenciaSQL.close();
    if (conexión != null) conexión.close();
  }

  public String[] tablas() throws java.sql.SQLException
  {
    String[] tablas = new String[3];
    tablas[0] = "alumnos";
    tablas[1] = "asignaturas";
    tablas[2] = "alumnos_asignaturas";
    return tablas;
  }

  public java.sql.ResultSet obtenerTabla(String tabla)
      throws java.sql.SQLException
  {
    cdr = sentenciaSQL.executeQuery(
        "SELECT * FROM " + tabla);
    return cdr;
  }
  
  public java.sql.ResultSet buscarFilasEnAlumnos(String subcad,
      int tipoBúsqueda) throws java.sql.SQLException
  {
    String[] cadena = {"'" + subcad + "%'", "'%" + subcad + "%'"};
    cdr = sentenciaSQL.executeQuery(
        "SELECT * FROM " + "alumnos" +
           " WHERE apellidos LIKE " + cadena[tipoBúsqueda-1]);
    return cdr;
  }
  
  public void insertarFilaEnAlumnos(int id_alumno,
      String apellidos, String nombre, int curso, int titulación)
      throws java.sql.SQLException
  {
    sentenciaSQL.executeUpdate("INSERT INTO " + "alumnos" +
      " VALUES (" + id_alumno + ", '" + apellidos + "', '" +
            nombre + "', " + curso + ", " + titulación + ")"
     );
  }

  public void borrarFilaEnAlumnos(int ID)
      throws java.sql.SQLException
  {
    sentenciaSQL.executeUpdate("DELETE FROM " + "alumnos" +
       " WHERE id_alumno = " + ID);
  }

  public void mostrarTabla(String tabla)
      throws java.sql.SQLException
  {
    cdr = obtenerTabla(tabla);
    while(cdr.next()) mostrarFilaActual();
  }
  
  public void mostrarFilasDeAlumnos(String subcad, int tipoBusqueda)
      throws java.sql.SQLException
  {
    cdr = buscarFilasEnAlumnos(subcad, tipoBusqueda);
    while(cdr.next()) mostrarFilaActual();
  }
  
  public void mostrarFilaActual() throws java.sql.SQLException
  {
    int nColumnas = cdr.getMetaData().getColumnCount();
    for (int i = 1; i <= nColumnas; ++i)
    {
      System.out.print(cdr.getString(i) + " ");
    }
    System.out.println();
  }
  
  public void siguiente() throws java.sql.SQLException
  {
    if (!cdr.isLast()) cdr.next();
  }
  
  public void anterior() throws java.sql.SQLException
  {
    if (cdr.isBeforeFirst()) cdr.first();
    if (!cdr.isFirst()) cdr.previous();
  }
  
  public void primero() throws java.sql.SQLException
  {
    cdr.first();
  }
  
  public void último() throws java.sql.SQLException
  {
    cdr.last();
  }
}
  

