/*
 * SvTutorias.java
 *
 * autor Fco. Javier Ceballos
 */

import java.io.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;

import javax.sql.*;    // para DataSource
import javax.naming.*; // para Context

public class SvTutorias extends HttpServlet
{
  private DataSource servicioConexiones;

  public void init(ServletConfig config) throws ServletException
  {
    super.init(config);
    try
    {
      // Obtener el contexto de nombres de nuestro entorno
      Context initCtx = new InitialContext();
      Context envCtx = (Context) initCtx.lookup("java:comp/env");
      // Conexiones para la fuente de datos buscada
      servicioConexiones = (DataSource) envCtx.lookup("jdbc/bd_tutorias");
    }
    catch (Exception e)
    {
      throw new ServletException(
         "Imposible recuperar java:comp/env/jdbc/bd_tutorias", e);
    }
  }

  protected void doGet(HttpServletRequest request,
                       HttpServletResponse response)
  throws ServletException, IOException
  {
    // Tipo de la respuesta que ser enviada al cliente
    response.setContentType("text/html");
    // Obtener el objeto 'PrintWriter' para devolver la respuesta
    PrintWriter out = response.getWriter();
    mostrarFormulario(out);
  }
  
  protected void doPost(HttpServletRequest request,
                        HttpServletResponse response)
  throws ServletException, IOException
  {
    // Tipo de la respuesta que ser enviada al cliente
    response.setContentType("text/html");
    // Obtener el objeto 'PrintWriter' para devolver la respuesta
    PrintWriter out = response.getWriter();
    
    BDTutorias BD = null;    
    // Tomar los datos recibidos del cliente y escribirlos
    // en la base de datos.
    try
    {
      BD = new BDTutorias(servicioConexiones);
      // Obtener los datos enviados por el cliente.
      String sID = request.getParameter("idtutoria");
      if (sID == null || sID.length() == 0) return;
      int id_tutoria = Integer.parseInt(sID);
      String alumno = request.getParameter("alumno");
      String asunto = request.getParameter("asunto");
      // Insertar una fila en la tabla citas
      BD.insertarEnCitas(id_tutoria, alumno, asunto);

      // Responder al cliente
      out.println("<html>");
      out.println("<title>Respuesta a la solicitud</title>");
      out.println("Su peticin ha sido registrada<br>Un saludo");
      out.println("</html>");
    }
    catch(SQLException e)
    { 
      out.println("<html>");
      out.println("<h3>Imposible realizar la operacin</h3>");
      // Invocar de nuevo al servlet para actualizar la pgina
      // mostrada por el explorador
      out.println("<p><a href='SvTutorias'>continuar</a></p>");
      out.println("</html>");
      // Si no se hace clic en "continuar" realizar la operacin
      // anterior transcurridos 5 segundos
      response.setHeader("Refresh", "5; URL=SvTutorias");
    }
    catch(Exception e)
    {
      out.println("Error: " + e.getMessage());
    }
    finally  // pase lo que pase devolvemos la conexin
    {
      try
      {
        if (BD != null) BD.cerrarConexion();
      }
      catch(Exception e)
      {
        System.out.println("Error en proceso " + e.getMessage());
      }
    }
    out.close();
  }
  
  public void mostrarFormulario(PrintWriter out)
  {
    BDTutorias BD = null;    
    try
    {
      BD = new BDTutorias(servicioConexiones);
      ResultSet cdr = BD.tutoriasDisponibles();
      // Formulario
      out.println("<html>" +
      "<head><title>Concertar una tutora</title></head>" +
      "<body><center>" +
        "<h1><font face='Arial'>CONCERTAR UNA TUTORA</font></h1>");
        if (cdr.isBeforeFirst() == false) // no hay filas
        {
          out.println("<br>TODAS LAS TUTORAS ESTN CONCERTADAS" +
                      "<br></center></body></html>");
          return;
        }
        out.println("<form action='http://localhost:8080/" +
                    "tutorias/SvTutorias'" +
                    "method=post>" +
        "<font face='Arial'>Alumno:</font>&nbsp;&nbsp;&nbsp;&nbsp;" +
        "<input type='text' name='alumno' size='60'>" +
        "<br><br><font face='Arial'>ID tutora:</font>" +
        "&nbsp;&nbsp;<select name='idtutoria'>");
        while (cdr.next())
          out.println("<option>" + cdr.getString("id_tutoria"));
        out.println("</select><br><br>");
        mostrarTutoriasDisponibles(cdr, out);
        out.println("<br><font face='Arial'>Asunto:</font>" +
        "<br><textarea name='asunto' rows='5' cols='53' wrap></textarea>" +
        "<br><br><input type='submit' value='Enviar datos'>" +
        "&nbsp;&nbsp;<input type='reset'>" +
      "</form></center></body></html>");
    }
    catch(SQLException e)
    {
      out.println("Excepcin SQL. " + e.getMessage());
    }
    catch(Exception e)
    {
      out.println("Error: " + e.getMessage());
    }
    finally  // pase lo que pase devolvemos la conexin
    {
      try
      {
        if (BD != null) BD.cerrarConexion();
      }
      catch(Exception e)
      {
        System.out.println("Error en proceso " + e.getMessage());
      }
    }
    out.close();
  }
  
  public void mostrarTutoriasDisponibles(ResultSet cdr, PrintWriter out)
  {
    try
    {
      // Tabla
      out.println("<table width=100% border=1>");
      // Cabeceras
      out.println(
        "<tr>" +
          "<th>ID tutora</th>" +
          "<th>Da/hora</th>" +
          "<th>Con el profesor</th>" +
        "</tr>");
      // Filas
      cdr.beforeFirst();
      while (cdr.next())
      {
        out.println(
          "<tr>" + 
            "<td width=10%>" + cdr.getString("id_tutoria") + "</td>" +
            "<td width=30%>" + cdr.getString("dia") + " a las " +
                               cdr.getString("hora") + "</td>" +
            "<td width=60%>" + cdr.getString("profesor") + "</td>" +
          "</tr>");
      } 
      out.println("</table>");
    }
    catch(SQLException e)
    {
      out.println("Excepcin SQL. " + e.getMessage());
    }
  }

  // Devuelve una descripcin breve.
  public String getServletInfo()
  {
    return "Servlet SvTutorias";
  }
}
