/*
 * SvTutorias.java
 *
 * autor Fco. Javier Ceballos
 */

import java.io.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import beans.*;

import javax.sql.*;    // para DataSource
import javax.naming.*; // para Context

public class SvTutorias extends HttpServlet
{
  private DataSource servicioConexiones;

  public void init(ServletConfig config) throws ServletException
  {
    super.init(config);
    try
    {
      // Obtener el contexto de nombres de nuestro entorno
      Context initCtx = new InitialContext();
      Context envCtx = (Context) initCtx.lookup("java:comp/env");
      // Conexiones para la fuente de datos buscada
      servicioConexiones = (DataSource) envCtx.lookup("jdbc/bd_tutorias");
    }
    catch (Exception e)
    {
      throw new ServletException(
         "Imposible recuperar java:comp/env/jdbc/bd_tutorias", e);
    }
  }

  protected void doGet(HttpServletRequest request,
                       HttpServletResponse response)
  throws ServletException, IOException
  {
    BeanBDTutorias bBDTut = null;
    try
    {
      // Realizar una conexin con la BD
      bBDTut = new BeanBDTutorias(servicioConexiones);
      // Consulta para obtener las tutoras disponibles
      TreeSet<BeanDatosTutoria> respuesta = bBDTut.getTutoriasDisponibles();
      int nFilas = respuesta.size();
      // Poner los datos obtenidos como atributos en la peticin
      request.setAttribute("tutoriasDisponibles", respuesta);
      request.setAttribute("numFilas", Integer.toString(nFilas));
      // Reenviar la peticin a la pgina JSP especificada
      RequestDispatcher dispatcher =
                request.getRequestDispatcher("/WEB-INF/MostrarForm.jsp");
      if (dispatcher != null)
        dispatcher.forward(request, response);
    }
    catch(SQLException e)
    {
      System.out.println("Excepcin SQL. " + e.getMessage());
    }
    catch(Exception e)
    {
      System.out.println("Error: " + e.getMessage());
    }
    finally  // pase lo que pase devolvemos la conexin
    {
      try
      {
        bBDTut.cerrarConexion();
      }
      catch(SQLException ignorada){}
    }
  }

  protected void doPost(HttpServletRequest request,
                        HttpServletResponse response)
  throws ServletException, IOException
  {
    BeanBDTutorias bBDTut = null;
    String operFallida = new String("SI");
    try
    {
      // Obtener los datos enviados por el cliente.
      String sID = request.getParameter("idtutoria");
      if (sID == null || sID.length() == 0) return;
      int id_tutoria = Integer.parseInt(sID);
      String alumno = request.getParameter("alumno");
      String asunto = request.getParameter("asunto");
      // Realizar una conexin con la BD
      bBDTut = new BeanBDTutorias(servicioConexiones);
      // Insertar una fila en la tabla citas
      bBDTut.insertarEnCitas(id_tutoria, alumno, asunto);
      operFallida = new String("NO");
    }
    catch(SQLException e)
    { 
      System.out.println("Error SQL: " + e.getMessage());
    }
    catch(Exception e)
    {
      System.out.println("Error: " + e.getMessage());
    }
    finally  // pase lo que pase devolvemos la conexin
    {
      try
      {
        bBDTut.cerrarConexion();
      }
      catch(SQLException ignorada){}
    }
    // Responder al cliente
    request.setAttribute("operacionFallida", operFallida);
    RequestDispatcher dispatcher =
              request.getRequestDispatcher("/WEB-INF/InsertarFila.jsp");
    if (dispatcher != null)
      dispatcher.forward(request, response);
  }

  // Devuelve una descripcin breve.
  public String getServletInfo()
  {
    return "Servlet SvTutorias";
  }
}
